@@ -66,9 +66,12 @@ struct _RsttoMainWindowPriv
         gboolean        toolbar_visibility;
         gint            max_cache_size;
         gdouble         slideshow_timeout;
+        gboolean        slideshow_hide_thumbnail;
         const GdkColor *bg_color;
         gboolean        scale_to_100;
         RsttoDesktop    desktop;
+        gboolean        start_fullscreen;
+        gboolean        start_slideshow;
     } settings;
 
     struct {
@@ -88,6 +91,7 @@ struct _RsttoMainWindowPriv
             GtkWidget *menu_item_file_properties;
             GtkWidget *menu_item_separator_2;
             GtkWidget *menu_item_close;
+            GtkWidget *menu_item_close_all;
             GtkWidget *menu_item_quit;
         } file;
 
@@ -248,6 +252,8 @@ cb_rstto_main_window_clear_recent(GtkWidget *widget, RsttoMainWindow *window);
 static void
 cb_rstto_main_window_close(GtkWidget *widget, RsttoMainWindow *window);
 static void
+cb_rstto_main_window_close_all(GtkWidget *widget, RsttoMainWindow *window);
+static void
 cb_rstto_main_window_file_properties(GtkWidget *widget, RsttoMainWindow *window);
 static void
 cb_rstto_main_window_quit(GtkWidget *widget, RsttoMainWindow *window);
@@ -343,6 +349,7 @@ rstto_main_window_init(RsttoMainWindow *window)
     window->priv->menus.file.menu_item_file_properties = gtk_image_menu_item_new_from_stock(GTK_STOCK_PROPERTIES, accel_group);
     window->priv->menus.file.menu_item_separator_2 = gtk_separator_menu_item_new();
     window->priv->menus.file.menu_item_close = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLOSE, accel_group);
+    window->priv->menus.file.menu_item_close_all = gtk_menu_item_new_with_mnemonic(_("Close _all"));
     window->priv->menus.file.menu_item_quit = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, accel_group);
 
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(window->priv->menus.menu_item_file), window->priv->menus.file.menu);
@@ -353,9 +360,11 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_file_properties);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_separator_2);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_close);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_close_all);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.file.menu), window->priv->menus.file.menu_item_quit);
 
     gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close, FALSE);
+    gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close_all, FALSE);
     gtk_widget_set_sensitive(window->priv->menus.file.menu_item_file_properties, FALSE);
 
     window->priv->menus.file.recently.menu = gtk_recent_chooser_menu_new_for_manager(GTK_RECENT_MANAGER(window->priv->manager));
@@ -782,6 +791,9 @@ rstto_main_window_init(RsttoMainWindow *window)
     g_signal_connect(window->priv->menus.file.menu_item_close, 
             "activate",
             G_CALLBACK(cb_rstto_main_window_close), window);
+    g_signal_connect(window->priv->menus.file.menu_item_close_all, 
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_close_all), window);
     g_signal_connect(G_OBJECT(window->priv->menus.file.recently.menu),
             "item-activated",
             G_CALLBACK(cb_rstto_main_window_open_recent), window);
@@ -1044,6 +1056,12 @@ rstto_main_window_get_slideshow_timeout (RsttoMainWindow *window)
     return window->priv->settings.slideshow_timeout;
 }
 
+gboolean
+rstto_main_window_get_hide_thumbnail (RsttoMainWindow *window)
+{
+    return window->priv->settings.slideshow_hide_thumbnail;
+}
+
 gint
 rstto_main_window_get_max_cache_size (RsttoMainWindow *window)
 {
@@ -1057,6 +1075,37 @@ rstto_main_window_set_slideshow_timeout (RsttoMainWindow *window, gdouble timeou
     rstto_navigator_set_timeout(window->priv->navigator, timeout);
 }
 
+void
+rstto_main_window_set_hide_thumbnail (RsttoMainWindow *window, gboolean hide)
+{
+    window->priv->settings.slideshow_hide_thumbnail = hide;
+}
+
+void
+rstto_main_window_set_start_fullscreen (RsttoMainWindow *window, gboolean fullscreen)
+{
+    window->priv->settings.start_fullscreen = fullscreen;
+}
+
+void
+rstto_main_window_set_start_slideshow (RsttoMainWindow *window, gboolean slideshow)
+{
+    window->priv->settings.start_slideshow = slideshow;
+}
+
+void
+rstto_main_window_force_fullscreen (RsttoMainWindow *window)
+{
+    if(gdk_window_get_state(GTK_WIDGET(window)->window) | GDK_WINDOW_STATE_FULLSCREEN)
+        gtk_window_fullscreen(GTK_WINDOW(window));
+}
+
+void
+rstto_main_window_force_slideshow (RsttoMainWindow *window)
+{
+    gtk_widget_activate(window->priv->menus.go.menu_item_play);
+}
+
 void
 rstto_main_window_set_max_cache_size (RsttoMainWindow *window, gint max_cache_size)
 {
@@ -1296,6 +1345,9 @@ cb_rstto_main_window_play(GtkWidget *widget, RsttoMainWindow *window)
     gtk_menu_shell_insert(GTK_MENU_SHELL(window->priv->menus.go.menu), window->priv->menus.go.menu_item_pause, 5);
     gtk_widget_show_all(window->priv->menus.go.menu_item_pause);
     rstto_navigator_set_running(RSTTO_NAVIGATOR(window->priv->navigator), TRUE);
+    if (window->priv->settings.thumbnail_viewer_visibility &&
+        window->priv->settings.slideshow_hide_thumbnail)
+            gtk_widget_hide (window->priv->thumbnail_viewer);
 }
 
 static void
@@ -1306,6 +1358,8 @@ cb_rstto_main_window_pause(GtkWidget *widget, RsttoMainWindow *window)
     gtk_menu_shell_insert(GTK_MENU_SHELL(window->priv->menus.go.menu), window->priv->menus.go.menu_item_play, 5);
     gtk_widget_show_all(window->priv->menus.go.menu_item_play);
     rstto_navigator_set_running(RSTTO_NAVIGATOR(window->priv->navigator), FALSE);
+    if (window->priv->settings.thumbnail_viewer_visibility)
+        gtk_widget_show (window->priv->thumbnail_viewer);
 }
 
 static void
@@ -1434,7 +1488,11 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     GtkWidget *preload_vbox = gtk_vbox_new(FALSE, 0);
     GtkWidget *preload_frame = xfce_create_framebox_with_content (_("Preload"), preload_vbox);
 
+    GtkWidget *thumbnails_vbox = gtk_vbox_new(FALSE, 0);
+    GtkWidget *thumbnails_frame = xfce_create_framebox_with_content (_("Thumbnails"), thumbnails_vbox);
+
     gtk_container_set_border_width (GTK_CONTAINER (slideshow_frame), 8);
+    gtk_container_set_border_width (GTK_CONTAINER (thumbnails_frame), 8);
     gtk_container_set_border_width (GTK_CONTAINER (preload_frame), 8);
 
     GtkWidget *slideshow_lbl = gtk_label_new(_("The time period an individual image is displayed during a slideshow\n(in seconds)"));
@@ -1444,24 +1502,35 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
     GtkWidget *preload_lbl = gtk_label_new(_("Preload images during slideshow\n(uses more memory)"));
     GtkWidget *preload_check = gtk_check_button_new_with_mnemonic(_("_Preload images"));
 
+    GtkWidget *thumbnails_lbl = gtk_label_new(_("Hide the thumbnails bar during slideshow"));
+    GtkWidget *thumbnails_check = gtk_check_button_new_with_mnemonic(_("_Hide thumbnails"));
+
     gtk_misc_set_alignment(GTK_MISC(slideshow_lbl), 0, 0.5);
     gtk_misc_set_alignment(GTK_MISC(preload_lbl), 0, 0.5);
+    gtk_misc_set_alignment(GTK_MISC(thumbnails_lbl), 0, 0.5);
 
     gtk_misc_set_padding(GTK_MISC(slideshow_lbl), 2, 2);
     gtk_misc_set_padding(GTK_MISC(preload_lbl), 2, 2);
+    gtk_misc_set_padding(GTK_MISC(thumbnails_lbl), 2, 2);
     
     gtk_range_set_value(GTK_RANGE(slideshow_hscale), window->priv->settings.slideshow_timeout / 1000);
 
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(preload_check), window->priv->navigator->preload);
 
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(thumbnails_check), window->priv->settings.slideshow_hide_thumbnail);
+
     gtk_box_pack_start(GTK_BOX(slideshow_vbox), slideshow_lbl, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(slideshow_vbox), slideshow_hscale, FALSE, TRUE, 0);
 
     gtk_box_pack_start(GTK_BOX(preload_vbox), preload_lbl, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(preload_vbox), preload_check, FALSE, TRUE, 0);
 
+    gtk_box_pack_start(GTK_BOX(thumbnails_vbox), thumbnails_lbl, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(thumbnails_vbox), thumbnails_check, FALSE, TRUE, 0);
+
     gtk_box_pack_start(GTK_BOX(slideshow_main_vbox), slideshow_frame, FALSE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(slideshow_main_vbox), preload_frame, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(slideshow_main_vbox), thumbnails_frame, FALSE, TRUE, 0);
 
     gtk_widget_show_all(notebook);
 
@@ -1472,6 +1541,7 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
         case GTK_RESPONSE_OK:
             rstto_main_window_set_slideshow_timeout(window, gtk_range_get_value(GTK_RANGE(slideshow_hscale)) * 1000);
             window->priv->navigator->preload = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(preload_check));
+            window->priv->settings.slideshow_hide_thumbnail = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(thumbnails_check));
             if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(bg_color_override_check)) == TRUE)
             {
                 GdkColor *new_color = g_new0(GdkColor, 1);
@@ -1679,14 +1749,24 @@ cb_rstto_main_window_close(GtkWidget *widget, RsttoMainWindow *window)
         if (rstto_navigator_get_n_files(window->priv->navigator) == 0)
         {
             gtk_widget_set_sensitive(widget, FALSE);
+            gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close_all, FALSE);
         }
     }
     else
     {
         gtk_widget_set_sensitive(widget, FALSE);
+        gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close_all, FALSE);
     }
 }
 
+static void
+cb_rstto_main_window_close_all(GtkWidget *widget, RsttoMainWindow *window)
+{
+    rstto_navigator_clear (window->priv->navigator);
+    gtk_widget_set_sensitive(widget, FALSE);
+    gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close, FALSE);
+}
+
 static void
 cb_rstto_main_window_file_properties(GtkWidget *widget, RsttoMainWindow *window)
 {
@@ -1725,6 +1805,7 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
         info = rstto_navigator_entry_get_info(entry);
         filename = info->display_name;
         gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close, TRUE);
+        gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close_all, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.file.menu_item_file_properties, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_first, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_last, TRUE);
@@ -1842,6 +1923,7 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
     else
     {
         gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close, FALSE);
+        gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close_all, FALSE);
         gtk_widget_set_sensitive(window->priv->menus.view.menu_item_set_wallpaper, FALSE);
         gtk_window_set_title(GTK_WINDOW(window), PACKAGE_STRING);
         if (rstto_navigator_get_n_files(window->priv->navigator) == 0)
