@@ -88,17 +88,30 @@ rstto_main_window_get_recent_manager (RsttoMainWindow *window);
 
 gdouble
 rstto_main_window_get_slideshow_timeout (RsttoMainWindow *window);
+gboolean
+rstto_main_window_get_hide_thumbnails (RsttoMainWindow *window);
 gint
 rstto_main_window_get_max_cache_size (RsttoMainWindow *window);
 void
 rstto_main_window_set_slideshow_timeout (RsttoMainWindow *window, gdouble timeout);
 void
+rstto_main_window_set_hide_thumbnails (RsttoMainWindow *window, gboolean hide);
+void
 rstto_main_window_set_max_cache_size (RsttoMainWindow *window, gint max_cache_size);
 void
 rstto_main_window_set_pv_bg_color (RsttoMainWindow *window, const GdkColor *color);
 const GdkColor *
 rstto_main_window_get_pv_bg_color (RsttoMainWindow *window);
 
+void
+rstto_main_window_set_start_fullscreen (RsttoMainWindow *window, gboolean fullscreen);
+void
+rstto_main_window_set_start_slideshow (RsttoMainWindow *window, gboolean slideshow);
+void
+rstto_main_window_force_fullscreen (RsttoMainWindow *window);
+void
+rstto_main_window_force_slideshow (RsttoMainWindow *window);
+
 void
 rstto_main_window_set_scale_to_100 (RsttoMainWindow *window, gboolean scale_to_100);
 gboolean
@@ -110,6 +123,12 @@ gint
 rstto_main_window_get_desktop(RsttoMainWindow *window);
 gint
 rstto_main_window_set_desktop(RsttoMainWindow *window, RsttoDesktop desktop);
+gboolean
+rstto_main_window_get_hide_thumbnail (RsttoMainWindow *window);
+void
+rstto_main_window_set_hide_thumbnail (RsttoMainWindow *window, gboolean hide);
+void
+rstto_main_window_set_start_fullscreen (RsttoMainWindow *window, gboolean fullscreen);
 
 G_END_DECLS
 
