@@ -31,6 +31,9 @@ rstto_navigator_class_init(RsttoNavigatorClass *);
 static void
 rstto_navigator_dispose(GObject *object);
 
+static void
+rstto_navigator_entry_clear (RsttoNavigatorEntry *nav_entry);
+
 static gboolean
 cb_rstto_navigator_running(RsttoNavigator *navigator);
 
@@ -104,6 +107,8 @@ struct _RsttoNavigatorEntry
     GdkPixbufRotation    rotation;
     gboolean             h_flipped;
     gboolean             v_flipped;
+    gint                 x_offset;
+    gint                 y_offset;
 };
 
 
@@ -293,8 +298,11 @@ rstto_navigator_guard_history(RsttoNavigator *navigator, RsttoNavigatorEntry *en
     }
 
     /* add image to the cache-history */
-    navigator->history = g_list_prepend(navigator->history, entry);
-    
+    if (g_list_index(navigator->history, entry) == -1)
+    {
+        navigator->history = g_list_prepend(navigator->history, entry);
+    }
+
     GList *iter = NULL;
     guint64 size = 0;
 
@@ -308,42 +316,21 @@ rstto_navigator_guard_history(RsttoNavigator *navigator, RsttoNavigatorEntry *en
 
             if (size > navigator->max_history)
             {
-                if(nav_entry->thumb)
+                if (iter != navigator->file_iter)
                 {
-                    gdk_pixbuf_unref(nav_entry->thumb);
-                    nav_entry->thumb = NULL;
+                    rstto_navigator_entry_clear (nav_entry);
+                    iter = g_list_previous(iter);
                 }
-
-                if(nav_entry->io_channel)
+                else
                 {
-                    g_source_remove(nav_entry->io_source_id);
-                    g_io_channel_unref(nav_entry->io_channel);
-                    nav_entry->io_channel = NULL;
-                    nav_entry->io_source_id = 0;
+                    iter = g_list_previous(iter);
+                    nav_entry = iter->data;
+                    if (nav_entry)
+                    {
+                        rstto_navigator_entry_clear (nav_entry);
+                    }
+                    iter = g_list_next(iter);
                 }
-
-                if (entry->timeout_id)
-                {
-                    g_source_remove(entry->timeout_id);
-                    entry->timeout_id = 0;
-                }
-
-                if(nav_entry->loader)
-                {
-                    g_signal_handlers_disconnect_by_func(nav_entry->loader , cb_rstto_navigator_entry_area_prepared, nav_entry);
-                    gdk_pixbuf_loader_close(nav_entry->loader, NULL);
-                }
-                if(nav_entry->animation)
-                {
-                    g_object_unref(nav_entry->animation);
-                    nav_entry->animation = NULL;
-                }
-                if(nav_entry->src_pixbuf)
-                {
-                    gdk_pixbuf_unref(nav_entry->src_pixbuf);
-                    nav_entry->src_pixbuf = NULL;
-                }
-                iter = g_list_previous(iter);
                 navigator->history = g_list_remove(navigator->history, nav_entry);
             }
         }
@@ -515,28 +502,31 @@ rstto_navigator_add (RsttoNavigator *navigator, RsttoNavigatorEntry *entry, gboo
 {
     g_return_val_if_fail(navigator == entry->navigator, -1);
 
-    navigator->file_list = g_list_insert_sorted(navigator->file_list, entry, navigator->compare_func);
-    if (!navigator->file_iter)
+    if (g_list_index(navigator->file_list, entry) ==  -1)
     {
-        navigator->file_iter = navigator->file_list;
+        navigator->file_list = g_list_insert_sorted(navigator->file_list, entry, navigator->compare_func);
+        if (!navigator->file_iter)
+        {
+            navigator->file_iter = navigator->file_list;
+            if (navigator->busy == FALSE)
+            {
+                g_signal_emit(G_OBJECT(navigator),
+                          rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ITER_CHANGED],
+                          0,
+                          g_list_index(navigator->file_list, entry),
+                          entry,
+                          NULL);
+            }
+        }
+
+        if (with_monitor == TRUE)
+            entry->monitor_handle = thunar_vfs_monitor_add_file(navigator->monitor, entry->info->path, (ThunarVfsMonitorCallback)cb_rstto_navigator_entry_fs_event, entry);
+
         if (navigator->busy == FALSE)
         {
-            g_signal_emit(G_OBJECT(navigator),
-                      rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_ITER_CHANGED],
-                      0,
-                      g_list_index(navigator->file_list, entry),
-                      entry,
-                      NULL);
+            g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_NEW_ENTRY], 0, g_list_index(navigator->file_list, entry), entry, NULL);
         }
     }
-
-    if (with_monitor == TRUE)
-        entry->monitor_handle = thunar_vfs_monitor_add_file(navigator->monitor, entry->info->path, (ThunarVfsMonitorCallback)cb_rstto_navigator_entry_fs_event, entry);
-
-    if (navigator->busy == FALSE)
-    {
-        g_signal_emit(G_OBJECT(navigator), rstto_navigator_signals[RSTTO_NAVIGATOR_SIGNAL_NEW_ENTRY], 0, g_list_index(navigator->file_list, entry), entry, NULL);
-    }
     return g_list_index(navigator->file_list, entry);
 }
 
@@ -702,66 +692,73 @@ rstto_navigator_entry_new (RsttoNavigator *navigator, ThunarVfsInfo *info)
     gchar *filename = thunar_vfs_path_dup_string(info->path);
     if(filename)
     {
-        entry = g_new0(RsttoNavigatorEntry, 1);
+        GList *iter = g_list_find_custom(navigator->file_list, info->path, (GCompareFunc)cb_rstto_navigator_entry_path_compare_func);
+        if (iter)
+            entry = iter->data;
 
-        entry->info = info;
-        entry->exif_data = exif_data_new_from_file(filename);
-        entry->navigator = navigator;
-        
-        ExifEntry *exifentry = exif_data_get_entry(entry->exif_data, EXIF_TAG_ORIENTATION);
-        if (exifentry)
+        if (entry == NULL)
         {
-            gchar *val = g_new0(gchar, 20);
-            exif_entry_get_value(exifentry, val, 20);
-            if (!strcmp(val, "top - left"))
+            entry = g_new0(RsttoNavigatorEntry, 1);
+
+            entry->info = info;
+            entry->exif_data = exif_data_new_from_file(filename);
+            entry->navigator = navigator;
+            
+            ExifEntry *exifentry = exif_data_get_entry(entry->exif_data, EXIF_TAG_ORIENTATION);
+            if (exifentry)
             {
-                entry->v_flipped = FALSE;
-                entry->h_flipped = FALSE;
-                entry->rotation = GDK_PIXBUF_ROTATE_NONE;
+                gchar *val = g_new0(gchar, 20);
+                exif_entry_get_value(exifentry, val, 20);
+                if (!strcmp(val, "top - left"))
+                {
+                    entry->v_flipped = FALSE;
+                    entry->h_flipped = FALSE;
+                    entry->rotation = GDK_PIXBUF_ROTATE_NONE;
+                }
+                if (!strcmp(val, "top - right"))
+                {
+                    entry->v_flipped = FALSE;
+                    entry->h_flipped = TRUE;
+                    entry->rotation = GDK_PIXBUF_ROTATE_NONE;
+                }
+                if (!strcmp(val, "bottom - left"))
+                {
+                    entry->v_flipped = TRUE;
+                    entry->h_flipped = FALSE;
+                    entry->rotation = GDK_PIXBUF_ROTATE_NONE;
+                }
+                if (!strcmp(val, "bottom - right"))
+                {
+                    entry->v_flipped = FALSE;
+                    entry->h_flipped = FALSE;
+                    entry->rotation = GDK_PIXBUF_ROTATE_UPSIDEDOWN;
+                }
+                if (!strcmp(val, "right - top"))
+                {
+                    entry->v_flipped = FALSE;
+                    entry->h_flipped = FALSE;
+                    entry->rotation = GDK_PIXBUF_ROTATE_CLOCKWISE;
+                }
+                if (!strcmp(val, "right - bottom"))
+                {
+                    entry->v_flipped = FALSE;
+                    entry->h_flipped = TRUE;
+                    entry->rotation = GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
+                }
+                if (!strcmp(val, "left - top"))
+                {
+                    entry->v_flipped = FALSE;
+                    entry->h_flipped = TRUE;
+                    entry->rotation = GDK_PIXBUF_ROTATE_CLOCKWISE;
+                }
+                if (!strcmp(val, "left - bottom"))
+                {
+                    entry->v_flipped = FALSE;
+                    entry->h_flipped = FALSE;
+                    entry->rotation = GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
+                }
+                g_free(val);
             }
-            if (!strcmp(val, "top - right"))
-            {
-                entry->v_flipped = FALSE;
-                entry->h_flipped = TRUE;
-                entry->rotation = GDK_PIXBUF_ROTATE_NONE;
-            }
-            if (!strcmp(val, "bottom - left"))
-            {
-                entry->v_flipped = TRUE;
-                entry->h_flipped = FALSE;
-                entry->rotation = GDK_PIXBUF_ROTATE_NONE;
-            }
-            if (!strcmp(val, "bottom - right"))
-            {
-                entry->v_flipped = FALSE;
-                entry->h_flipped = FALSE;
-                entry->rotation = GDK_PIXBUF_ROTATE_UPSIDEDOWN;
-            }
-            if (!strcmp(val, "right - top"))
-            {
-                entry->v_flipped = FALSE;
-                entry->h_flipped = FALSE;
-                entry->rotation = GDK_PIXBUF_ROTATE_CLOCKWISE;
-            }
-            if (!strcmp(val, "right - bottom"))
-            {
-                entry->v_flipped = FALSE;
-                entry->h_flipped = TRUE;
-                entry->rotation = GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
-            }
-            if (!strcmp(val, "left - top"))
-            {
-                entry->v_flipped = FALSE;
-                entry->h_flipped = TRUE;
-                entry->rotation = GDK_PIXBUF_ROTATE_CLOCKWISE;
-            }
-            if (!strcmp(val, "left - bottom"))
-            {
-                entry->v_flipped = FALSE;
-                entry->h_flipped = FALSE;
-                entry->rotation = GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE;
-            }
-            g_free(val);
         }
 
         g_free(filename);
@@ -790,24 +787,21 @@ rstto_navigator_entry_get_flip (RsttoNavigatorEntry *entry, gboolean horizontal)
         return entry->v_flipped;
 }
 
-void
-rstto_navigator_entry_free(RsttoNavigatorEntry *nav_entry)
+static void
+rstto_navigator_entry_clear (RsttoNavigatorEntry *nav_entry)
 {
     if(nav_entry->thumb)
     {
         gdk_pixbuf_unref(nav_entry->thumb);
         nav_entry->thumb = NULL;
     }
-    if(nav_entry->exif_data)
-    {   
-        exif_data_free(nav_entry->exif_data);
-        nav_entry->exif_data = NULL;
-    }
-    
+
     if(nav_entry->io_channel)
     {
-        g_io_channel_unref(nav_entry->io_channel);
         g_source_remove(nav_entry->io_source_id);
+        g_io_channel_unref(nav_entry->io_channel);
+        nav_entry->io_channel = NULL;
+        nav_entry->io_source_id = 0;
     }
 
     if (nav_entry->timeout_id)
@@ -824,11 +818,20 @@ rstto_navigator_entry_free(RsttoNavigatorEntry *nav_entry)
     if(nav_entry->animation)
     {
         g_object_unref(nav_entry->animation);
+        nav_entry->animation = NULL;
     }
     if(nav_entry->src_pixbuf)
     {
         gdk_pixbuf_unref(nav_entry->src_pixbuf);
+        nav_entry->src_pixbuf = NULL;
     }
+
+}
+
+void
+rstto_navigator_entry_free(RsttoNavigatorEntry *nav_entry)
+{
+    rstto_navigator_entry_clear (nav_entry);
     thunar_vfs_info_unref(nav_entry->info);
     g_free(nav_entry);
 }
@@ -840,6 +843,7 @@ rstto_navigator_entry_get_thumb(RsttoNavigatorEntry *entry, gint size)
     {
         if(!(gdk_pixbuf_get_width(entry->thumb) == size || gdk_pixbuf_get_height(entry->thumb) == size))
         {
+            /* TODO: rebuild thumbnail */
         }
     }
     else
@@ -980,16 +984,31 @@ rstto_navigator_entry_get_pixbuf (RsttoNavigatorEntry *entry)
 guint64
 rstto_navigator_entry_get_size (RsttoNavigatorEntry *entry)
 {
+    guint64 size = 0;
+    
+    gint width, height, n_channels;
+
     if (entry->src_pixbuf)
     {
-        gint width = gdk_pixbuf_get_width(entry->src_pixbuf);
-        gint height = gdk_pixbuf_get_height(entry->src_pixbuf);
+        width = gdk_pixbuf_get_width(entry->src_pixbuf);
+        height = gdk_pixbuf_get_height(entry->src_pixbuf);
 
-        gint n_channels = gdk_pixbuf_get_n_channels(entry->src_pixbuf);
+        n_channels = gdk_pixbuf_get_n_channels(entry->src_pixbuf);
 
-        return (guint64) width * height * n_channels;
+        size += (guint64) width * height * n_channels;
     }
-    return 0;
+    if (entry->thumb)
+    {
+        width = gdk_pixbuf_get_width(entry->thumb);
+        height = gdk_pixbuf_get_height(entry->thumb);
+
+        n_channels = gdk_pixbuf_get_n_channels(entry->thumb);
+        size += (guint64) width * height * n_channels;
+    }
+    
+    size += (guint64)sizeof (RsttoNavigatorEntry);
+
+    return size;
 }
 
 gboolean
@@ -1015,6 +1034,7 @@ rstto_navigator_entry_load_image (RsttoNavigatorEntry *entry, gboolean empty_cac
 
         g_io_channel_set_encoding(entry->io_channel, NULL, NULL);
         entry->io_source_id = g_io_add_watch(entry->io_channel, G_IO_IN | G_IO_PRI, (GIOFunc)cb_rstto_navigator_entry_read_file, entry);
+        g_free(path);
     }
     else
     {
@@ -1324,7 +1344,7 @@ rstto_navigator_entry_select (RsttoNavigatorEntry *entry)
 
 }
 
-gdouble
+guint64
 rstto_navigator_get_max_history_size (RsttoNavigator *navigator)
 {
     return navigator->max_history;
@@ -1345,7 +1365,7 @@ rstto_navigator_entry_get_position(RsttoNavigatorEntry *entry)
 }
 
 void
-rstto_navigator_set_max_history_size(RsttoNavigator *nav, gdouble size)
+rstto_navigator_set_max_history_size(RsttoNavigator *nav, guint64 size)
 {
     nav->max_history = size;
 }
@@ -1424,6 +1444,9 @@ rstto_navigator_open_file(RsttoNavigator *navigator, const gchar *path, gboolean
     file_media = thunar_vfs_mime_info_get_media(vfs_info->mime_info);
     if(!strcmp(file_media, "image"))
     {
+        RsttoNavigatorEntry *entry = rstto_navigator_entry_new(navigator, vfs_info);
+        rstto_navigator_add (navigator, entry, TRUE);
+
         if (open_folder == TRUE)
         {
             ThunarVfsPath *parent_vfs_path = thunar_vfs_path_get_parent(vfs_path);
@@ -1441,20 +1464,7 @@ rstto_navigator_open_file(RsttoNavigator *navigator, const gchar *path, gboolean
             g_free(parent_path);
             thunar_vfs_path_unref(parent_vfs_path);
 
-            GList *iter = g_list_find_custom(navigator->file_list, vfs_path, (GCompareFunc)cb_rstto_navigator_entry_path_compare_func);
-            if (iter != NULL)
-            {
-                rstto_navigator_entry_select((RsttoNavigatorEntry*)iter->data);
-            }
-            else
-            {
-                rstto_navigator_jump_first(navigator);
-            }
-        }
-        else
-        {
-            RsttoNavigatorEntry *entry = rstto_navigator_entry_new(navigator, vfs_info);
-            rstto_navigator_add (navigator, entry, TRUE);
+            rstto_navigator_entry_select(entry);
         }
     }
 
@@ -1463,6 +1473,7 @@ rstto_navigator_open_file(RsttoNavigator *navigator, const gchar *path, gboolean
     g_free(file_uri);
 
     g_free(file_media);
+    return 0;
 }
 
 gboolean
@@ -1503,7 +1514,7 @@ rstto_navigator_open_folder(RsttoNavigator *navigator, const gchar *path, gboole
 
     dir_path = thunar_vfs_path_dup_string(vfs_path);
 
-    g_object_add_weak_pointer(G_OBJECT(navigator), (gpointer *)&navigator);
+    g_object_add_weak_pointer(G_OBJECT(navigator), (gpointer)navigator);
 
     dir = g_dir_open(dir_path, 0, NULL);
 
@@ -1559,7 +1570,8 @@ rstto_navigator_open_folder(RsttoNavigator *navigator, const gchar *path, gboole
     }
 
     g_free(dir_uri);
-    g_object_remove_weak_pointer(G_OBJECT(navigator), (gpointer *)&navigator);
+    g_free(dir_path);
+    g_object_remove_weak_pointer(G_OBJECT(navigator), (gpointer)navigator);
     rstto_navigator_set_busy(navigator, FALSE);
     return TRUE;
 }
