@@ -12,6 +12,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  Drag-n-Drop support taken from Thunar, written by Benedict Meurer
  */
 
 #include <config.h>
@@ -31,6 +33,15 @@ typedef enum
     RSTTO_PICTURE_VIEWER_STATE_BOX_ZOOM
 } RsttoPictureViewerState;
 
+enum
+{
+    TARGET_TEXT_URI_LIST,
+};
+
+static const GtkTargetEntry drop_targets[] = {
+    {"text/uri-list", 0, TARGET_TEXT_URI_LIST},
+};
+
 
 struct _RsttoPictureViewerPriv
 {
@@ -107,6 +118,21 @@ cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEve
 static void
 cb_rstto_picture_viewer_popup_menu (RsttoPictureViewer *viewer, gboolean user_data);
 
+static gboolean
+rstto_picture_viewer_drag_drop (GtkWidget *widget,
+                                GdkDragContext *context,
+                                gint x,
+                                gint y,
+                                guint time);
+static gboolean
+rstto_picture_viewer_drag_motion (GtkWidget *widget,
+                                GdkDragContext *context,
+                                gint x,
+                                gint y,
+                                guint time);
+static void
+rstto_picture_viewer_drag_data_received();
+
 static GtkWidgetClass *parent_class = NULL;
 
 GType
@@ -150,13 +176,16 @@ rstto_picture_viewer_init(RsttoPictureViewer *viewer)
                            GDK_BUTTON_RELEASE_MASK |
                            GDK_BUTTON1_MOTION_MASK);
 
-    viewer->priv->show_border = TRUE;
+    viewer->priv->show_border = FALSE;
 
     g_signal_connect(G_OBJECT(viewer), "scroll_event", G_CALLBACK(cb_rstto_picture_viewer_scroll_event), NULL);
     g_signal_connect(G_OBJECT(viewer), "button_press_event", G_CALLBACK(cb_rstto_picture_viewer_button_press_event), NULL);
     g_signal_connect(G_OBJECT(viewer), "button_release_event", G_CALLBACK(cb_rstto_picture_viewer_button_release_event), NULL);
     g_signal_connect(G_OBJECT(viewer), "motion_notify_event", G_CALLBACK(cb_rstto_picture_viewer_motion_notify_event), NULL);
     g_signal_connect(G_OBJECT(viewer), "popup-menu", G_CALLBACK(cb_rstto_picture_viewer_popup_menu), NULL);
+
+    gtk_drag_dest_set(GTK_WIDGET(viewer), 0, drop_targets, G_N_ELEMENTS(drop_targets),
+                      GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE | GDK_ACTION_PRIVATE);
 }
 
 static void
@@ -174,9 +203,11 @@ rstto_picture_viewer_class_init(RsttoPictureViewerClass *viewer_class)
 
     widget_class->realize = rstto_picture_viewer_realize;
     widget_class->expose_event = rstto_picture_viewer_expose;
-
     widget_class->size_request = rstto_picture_viewer_size_request;
     widget_class->size_allocate = rstto_picture_viewer_size_allocate;
+    widget_class->drag_drop = rstto_picture_viewer_drag_drop;
+    widget_class->drag_motion = rstto_picture_viewer_drag_motion;
+    widget_class->drag_data_received = rstto_picture_viewer_drag_data_received;
 
     object_class->destroy = rstto_picture_viewer_destroy;
 
@@ -348,35 +379,78 @@ rstto_picture_viewer_paint(GtkWidget *widget)
             {
                 gdk_gc_set_foreground(gc,
                         &(widget->style->fg[GTK_STATE_SELECTED]));
-                gdouble x1, x2, y1, y2;
+                gdouble m_x1, m_x2, m_y1, m_y2;
 
                 if (viewer->priv->motion.x < viewer->priv->motion.current_x)
                 {
-                    x1 = viewer->priv->motion.x;
-                    x2 = viewer->priv->motion.current_x;
+                    m_x1 = viewer->priv->motion.x;
+                    m_x2 = viewer->priv->motion.current_x;
                 }
                 else
                 {
-                    x1 = viewer->priv->motion.current_x;
-                    x2 = viewer->priv->motion.x;
+                    m_x1 = viewer->priv->motion.current_x;
+                    m_x2 = viewer->priv->motion.x;
                 }
                 if (viewer->priv->motion.y < viewer->priv->motion.current_y)
                 {
-                    y1 = viewer->priv->motion.y;
-                    y2 = viewer->priv->motion.current_y;
+                    m_y1 = viewer->priv->motion.y;
+                    m_y2 = viewer->priv->motion.current_y;
                 }
                 else
                 {
-                    y1 = viewer->priv->motion.current_y;
-                    y2 = viewer->priv->motion.y;
+                    m_y1 = viewer->priv->motion.current_y;
+                    m_y2 = viewer->priv->motion.y;
                 }
+                if (m_y1 < y1)
+                    m_y1 = y1;
+                if (m_x1 < x1)
+                    m_x1 = x1;
+
+                if (m_x2 > x2 + x1)
+                    m_x2 = x2 + x1;
+                if (m_y2 > y2 + y1)
+                    m_y2 = y2 + y1;
+
+                if ((m_x2 - m_x1 >= 2) && (m_y2 - m_y1 >= 2))
+                {
+                    GdkPixbuf *sub = gdk_pixbuf_new_subpixbuf(pixbuf,
+                                                              m_x1-x1,
+                                                              m_y1-y1,
+                                                              m_x2-m_x1,
+                                                              m_y2-m_y1);
+                    if(sub)
+                    {
+                        sub = gdk_pixbuf_composite_color_simple(sub,
+                                                          m_x2-m_x1,
+                                                          m_y2-m_y1,
+                                                          GDK_INTERP_BILINEAR,
+                                                          200,
+                                                          200,
+                                                          widget->style->bg[GTK_STATE_SELECTED].pixel,
+                                                          widget->style->bg[GTK_STATE_SELECTED].pixel);
+
+                        gdk_draw_pixbuf(GDK_DRAWABLE(buffer),
+                                        gc,
+                                        sub,
+                                        0,0,
+                                        m_x1,
+                                        m_y1,
+                                        -1, -1,
+                                        GDK_RGB_DITHER_NONE,
+                                        0, 0);
+
+                        gdk_pixbuf_unref(sub);
+                        sub = NULL;
+                    }
+                }
+
                 gdk_draw_rectangle(GDK_DRAWABLE(buffer),
                                 gc,
                                 FALSE,
-                                x1,
-                                y1,
-                                x2 - x1,
-                                y2 - y1);
+                                m_x1,
+                                m_y1,
+                                m_x2 - m_x1,
+                                m_y2 - m_y1);
             }
 
             if(viewer->priv->show_border)
@@ -573,6 +647,7 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
             {
                 scale = 1.0;
                 rstto_navigator_entry_set_scale(entry, scale);
+                rstto_navigator_entry_set_fit_to_screen(entry, FALSE);
                 fit_to_screen = FALSE;
             }
             else
@@ -887,6 +962,8 @@ cb_rstto_picture_viewer_button_press_event (RsttoPictureViewer *viewer, GdkEvent
     {
         viewer->priv->motion.x = event->x;
         viewer->priv->motion.y = event->y;
+        viewer->priv->motion.current_x = event->x;
+        viewer->priv->motion.current_y = event->y;
         viewer->priv->motion.h_val = viewer->hadjustment->value;
         viewer->priv->motion.v_val = viewer->vadjustment->value;
 
@@ -990,67 +1067,69 @@ cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEve
                     gdouble h_scale = widget->allocation.width / box_width * scale;
                     gdouble v_scale = widget->allocation.height / box_height * scale;
 
-                    if (h_scale < v_scale)
+                    /** 
+                     * Check if the mouse has been moved (and there exists a box
+                     */
+                    if (box_height > 1 && box_width > 1)
                     {
-                        rstto_navigator_entry_set_scale(entry, h_scale);
-                        gdouble d_box_height = box_height * v_scale / h_scale;
-                        top_left_y -= (d_box_height - box_height) / 2;
-                        box_height = d_box_height;
-                    }
-                    else
-                    {
-                        rstto_navigator_entry_set_scale(entry, v_scale);
-                        gdouble d_box_width = box_width * h_scale / v_scale;
-                        top_left_x -= (d_box_width - box_width) / 2;
-                        box_width = d_box_width;
-                    }
+                        if (h_scale < v_scale)
+                        {
+                            rstto_navigator_entry_set_scale(entry, h_scale);
+                            gdouble d_box_height = box_height * v_scale / h_scale;
+                            top_left_y -= (d_box_height - box_height) / 2;
+                            box_height = d_box_height;
+                        }
+                        else
+                        {
+                            rstto_navigator_entry_set_scale(entry, v_scale);
+                            gdouble d_box_width = box_width * h_scale / v_scale;
+                            top_left_x -= (d_box_width - box_width) / 2;
+                            box_width = d_box_width;
+                        }
 
-                    rstto_navigator_entry_set_fit_to_screen(entry, FALSE);
-                    scale = rstto_navigator_entry_get_scale(entry);
+                        rstto_navigator_entry_set_fit_to_screen(entry, FALSE);
+                        scale = rstto_navigator_entry_get_scale(entry);
 
-
-                        
-                    if(viewer->hadjustment)
-                    {
-                        viewer->hadjustment->page_size = box_width / old_scale * scale;
-                        viewer->hadjustment->upper = width * scale;
-                        viewer->hadjustment->lower = 0;
-                        viewer->hadjustment->step_increment = 1;
-                        viewer->hadjustment->page_increment = 100;
-                        viewer->hadjustment->value = top_left_x / old_scale * scale;
-                        if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
+                        if(viewer->hadjustment)
                         {
-                            viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
+                            viewer->hadjustment->page_size = box_width / old_scale * scale;
+                            viewer->hadjustment->upper = width * scale;
+                            viewer->hadjustment->lower = 0;
+                            viewer->hadjustment->step_increment = 1;
+                            viewer->hadjustment->page_increment = 100;
+                            viewer->hadjustment->value = top_left_x / old_scale * scale;
+                            if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
+                            {
+                                viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
+                            }
+                            if(viewer->hadjustment->value < viewer->hadjustment->lower)
+                            {
+                                viewer->hadjustment->value = viewer->hadjustment->lower;
+                            }
+                            gtk_adjustment_changed(viewer->hadjustment);
+                            gtk_adjustment_value_changed(viewer->hadjustment);
                         }
-                        if(viewer->hadjustment->value < viewer->hadjustment->lower)
+                        if(viewer->vadjustment)
                         {
-                            viewer->hadjustment->value = viewer->hadjustment->lower;
+                            viewer->vadjustment->page_size = box_height /old_scale* scale;
+                            viewer->vadjustment->upper = height * scale;
+                            viewer->vadjustment->lower = 0;
+                            viewer->vadjustment->step_increment = 1;
+                            viewer->vadjustment->page_increment = 100;
+                            viewer->vadjustment->value = top_left_y / old_scale * scale;
+                            if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
+                            {
+                                viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
+                            }
+                            if(viewer->vadjustment->value < viewer->vadjustment->lower)
+                            {
+                                viewer->vadjustment->value = viewer->vadjustment->lower;
+                            }
+                            gtk_adjustment_changed(viewer->vadjustment);
+                            gtk_adjustment_value_changed(viewer->vadjustment);
                         }
-                        gtk_adjustment_changed(viewer->hadjustment);
-                        gtk_adjustment_value_changed(viewer->hadjustment);
-                    }
-                    if(viewer->vadjustment)
-                    {
-                        viewer->vadjustment->page_size = box_height /old_scale* scale;
-                        viewer->vadjustment->upper = height * scale;
-                        viewer->vadjustment->lower = 0;
-                        viewer->vadjustment->step_increment = 1;
-                        viewer->vadjustment->page_increment = 100;
-                        viewer->vadjustment->value = top_left_y / old_scale * scale;
-                        if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
-                        {
-                            viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
-                        }
-                        if(viewer->vadjustment->value < viewer->vadjustment->lower)
-                        {
-                            viewer->vadjustment->value = viewer->vadjustment->lower;
-                        }
-                        gtk_adjustment_changed(viewer->vadjustment);
-                        gtk_adjustment_value_changed(viewer->vadjustment);
                     }
                 }
-                viewer->priv->motion.state = RSTTO_PICTURE_VIEWER_STATE_NONE;
-
                 if (viewer->priv->refresh.idle_id > 0)
                 {
                     g_source_remove(viewer->priv->refresh.idle_id);
@@ -1061,6 +1140,8 @@ cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEve
 
     }
 
+    viewer->priv->motion.state = RSTTO_PICTURE_VIEWER_STATE_NONE;
+
 }
 
 static void
@@ -1130,3 +1211,96 @@ rstto_picture_viewer_set_zoom_mode(RsttoPictureViewer *viewer, RsttoZoomMode mod
 {
     viewer->priv->zoom_mode = mode;
 }
+
+static void
+rstto_picture_viewer_drag_data_received(GtkWidget *widget,
+                                        GdkDragContext *context,
+                                        gint x,
+                                        gint y,
+                                        GtkSelectionData *selection_data,
+                                        guint info,
+                                        guint time)
+{
+    RsttoPictureViewer *picture_viewer = RSTTO_PICTURE_VIEWER(widget);
+    gchar **array = gtk_selection_data_get_uris (selection_data);
+
+    context->action = GDK_ACTION_PRIVATE;
+
+    if (array == NULL)
+    {
+        gtk_drag_finish (context, FALSE, FALSE, time);
+    }
+
+    gchar **_array = array;
+
+    while(*_array)
+    {
+        ThunarVfsPath *vfs_path = thunar_vfs_path_new(*_array, NULL);
+        gchar *path = thunar_vfs_path_dup_string(vfs_path);
+        if (g_file_test(path, G_FILE_TEST_EXISTS))
+        {
+            if (g_file_test(path, G_FILE_TEST_IS_DIR))
+            {
+                if(rstto_navigator_open_folder(picture_viewer->priv->navigator, path, FALSE, NULL) == TRUE)
+                {
+                    rstto_navigator_jump_first(picture_viewer->priv->navigator);
+                }
+            }
+            else
+            {
+                rstto_navigator_open_file(picture_viewer->priv->navigator, path, FALSE, NULL);
+            }
+        }
+
+        g_free(path);
+        thunar_vfs_path_unref(vfs_path);
+        _array++;
+    }
+    
+    gtk_drag_finish (context, TRUE, FALSE, time);
+}
+
+static gboolean
+rstto_picture_viewer_drag_drop (GtkWidget *widget,
+                                GdkDragContext *context,
+                                gint x,
+                                gint y,
+                                guint time)
+{
+    GdkAtom target;
+
+    /* determine the drop target */
+    target = gtk_drag_dest_find_target (widget, context, NULL);
+    if (G_LIKELY (target == gdk_atom_intern ("text/uri-list", FALSE)))
+    {
+        /* set state so the drag-data-received handler
+         * knows that this is really a drop this time.
+         */
+
+        /* request the drag data from the source. */
+        gtk_drag_get_data (widget, context, target, time);
+    }
+    else
+    {
+        return FALSE;
+    }
+    return TRUE;
+}
+
+static gboolean
+rstto_picture_viewer_drag_motion (GtkWidget *widget,
+                                GdkDragContext *context,
+                                gint x,
+                                gint y,
+                                guint time)
+{
+    GdkAtom target;
+    target = gtk_drag_dest_find_target (widget, context, NULL);
+    if (G_UNLIKELY (target != gdk_atom_intern ("text/uri-list", FALSE)))
+    {
+        /* we cannot handle the drop */
+        g_debug("FAAAAAAAAAAAAAALSE");
+        return FALSE;
+    }
+    return TRUE;
+}
